import numpy as np
import matplotlib.pyplot as plt

class Mesh:
    def __init__(self, vertices=None, cells=None):
        if vertices is None:
            self.vertices = np.empty((0, 0))
        else:
            self.vertices = np.array(vertices)   # copy
            
        if cells is None:
            self.cells = np.empty((0, 0), dtype=np.int)
        else:
            self.cells = np.array(cells, dtype=np.int)   # copy
            
    def build_edges(self):
        # loops over the triangles and store the edges
        edges = np.empty((self.cells.shape[0] * 3, 2), dtype=np.int)
        for icell, cell in enumerate(self.cells):
            edges[3*icell] = cell[0:2]
            edges[3*icell + 1] = cell[1:3]
            edges[3*icell + 2] = cell[[2, 0]]
                      
        self.edges, counts = np.unique(np.sort(edges, axis=1), axis=0, return_counts=True)
        self.interior_edges = self.edges[counts == 2]
        self.boundary_edges = self.edges[counts == 1]
        
    def __repr__(self):
        plt.axis('equal')
        plt.plot(self.vertices[:, 0], self.vertices[:, 1], 'ko')
        ex = np.array([self.vertices[self.edges[:, 0], 0], self.vertices[self.edges[:, 1], 0]])
        ey = np.array([self.vertices[self.edges[:, 0], 1], self.vertices[self.edges[:, 1], 1]])
        plt.plot(ex, ey, 'k')
        plt.show()
        return f'Mesh with {self.cells.shape[0]} triangles and {self.vertices.shape[0]} vertices'

