class Polynomial:
    
    " Class representing a polynom P(x) -> c_0 + c_1*x + c_2*x^2 + ..."
    
    def __init__(self, coeffs):
        self.coeffs = coeffs
        
    def __call__(self, x):
        return sum([coef*x**exp for exp, coef in enumerate(self.coeffs)])
    
    def diff(self, n):
        coeffs = self.coeffs
        for k in range(n):
            coeffs = [i * coeffs[i] for i in range(1, len(coeffs))]
        return Polynomial(coeffs)
    
    def __repr__(self): 
        out = ""
        pf = "d"
        for k, c in enumerate(self.coeffs):
            if c != 0:
                if c == 1:
                    val = "+" if k != 0 else "1"
                elif c == -1:
                    val = "-" if k != 0 else "-1"
                else:
                    val = f"{c:{pf}}"
                    
                
                if k == 0:
                    out += val
                elif k == 1:
                    out += val + "X"
                else:
                    out += val + f"X^{k}"
                
                pf = "+d"
                
        return out
    
    
    def __eq__(self, other):
        return self.coeffs == other.coeffs

    def __add__(self, other):
        if len(self.coeffs) > len(other.coeffs):
            tmp = other.coeffs + [0] * (len(self.coeffs) - len(other.coeffs))
            return Polynomial([a + b for a, b in zip(self.coeffs, tmp)])
        else:
            tmp = self.coeffs + [0] * (len(other.coeffs) - len(self.coeffs))
            return Polynomial([a + b for a, b in zip(tmp, other.coeffs)])

    def __neg__(self):
        return Polynomial([-c for c in self.coeffs])
        
    def __sub__(self, other):
        return self.__add__(-other)

    def __mul__(self, other):
        if isinstance(other, Polynomial):
            pol = [0]*(len(self.coeffs) + len(other.coeffs))
            for i in range(len(self.coeffs)):
                for j in range(len(other.coeffs)):
                    pol[i+j] += self.coeffs[i] * other.coeffs[j]
            return Polynomial(pol)
        else:
            return Polynomial([other * c for c in self.coeffs])
        