import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation

fig = plt.figure()
axis = fig.add_subplot(111)

c = np.sqrt(2)/2

Nframe = 13
ri, gi, bi = 83/255, 53/255, 10/255
re, ge, be = 89/255, 168/255, 15/255
r = np.linspace(ri, re, Nframe)
g = np.linspace(gi, ge, Nframe)
b = np.linspace(bi, be, Nframe)


# initialization: first square
xy = np.array([(-0.5, 0)])
angles = np.array([0])
L = 1

# the first rectangle
patches = []
patches.append(plt.Rectangle(xy[0], L, L, angle=angles[0], ec=(ri, gi, bi), fc=(ri, gi, bi), lw=1))

# function to compute the others
def compute_branch(iframe):
    global xy, L, angles
    
    new_xy = np.empty(2*xy.shape[0], dtype=(np.float64, 2))
    new_angles = np.empty(2*angles.size)
    
    rad = angles * np.pi / 180
    v1 = np.array([-np.sin(rad), np.cos(rad)])
    v2 = np.array([np.cos(rad) * 0.5 - np.sin(rad) * 1.5, np.sin(rad) * 0.5 + np.cos(rad) * 1.5])
    
    for i in range(xy.shape[0]):
        pt = xy[i]
        new_xy[2*i] = (pt[0] + L * v1[0, i], pt[1] + L * v1[1, i])
        new_xy[2*i+1] = (pt[0] + L * v2[0, i], pt[1] + L * v2[1, i])
        
        new_angles[2*i] = (angles[i] + 45) % 360
        new_angles[2*i+1] = (angles[i] - 45) % 360
                
    xy = new_xy
    angles = new_angles
    L *= c
    
    patches = [plt.Rectangle(xy[i], L, L, angle=angles[i],
                                     ec=(r[iframe], g[iframe], b[iframe]), 
                                     fc=(r[iframe], g[iframe], b[iframe]), 
                             lw=1) for i in range(xy.shape[0])]
    return patches


# filling the tree
for i in range(1, Nframe):
    for patch in compute_branch(i):
        patches.append(patch)


# init function for animation
def init():
    global patches
    
    axis.cla()
    axis.set_aspect('equal')
    axis.set_xlim((-3, 3))
    axis.set_ylim((0, 6))
    
    for patch in patches:
        axis.add_patch(patch)
            
    for i in range(1, len(patches)):
        patches[i].set_visible(False)
        
    return patches


# updating function for the animation
def update(iframe):
    global patches

    for i in range(2**iframe - 1, 2**(iframe + 1) - 1):
        patches[i].set_visible(True)
        
    return patches  

# creates animation object
# The animation calls the function update to update the frame
# fig is the figure where to play the animation
# update is the function that is called to update the frame
# frame is an integer or any iterable object. 
#  - If frame is an integer, it is understood as the total number of frame and the input parameter of the function
#  update is the current frame number
#  - If frame is an iterable object, the input argument of update is the current iteration value. The animation stops
#  when the object has finished iterating.
# interval is the number of ms to wait before updating the frame

anim = animation.FuncAnimation(fig, update, frames=Nframe, init_func=init, 
                               interval=1, repeat=True, repeat_delay=200)
plt.show()
