import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation


# 2D random walk
N = 1000
tags = np.random.randint(0, 4, N)
steps = np.zeros((N, 2))
steps[tags == 0, 0] = -1
steps[tags == 1, 0] = 1
steps[tags == 2, 1] = -1
steps[tags == 3, 1] = 1

walk = np.cumsum(steps, axis=0)

fig = plt.figure()
ax = plt.subplot(111)

xmin, xmax = np.amin(walk[:, 0]), np.amax(walk[:, 0])
ymin, ymax = np.amin(walk[:, 1]), np.amax(walk[:, 1])
dx, dy = 0.02 * (xmax - xmin), 0.02 * (ymax - ymin)

cm = plt.get_cmap("coolwarm")

def init_frame():
    ax.cla()
    ax.set_xlim(xmin - dx, xmax + dx)
    ax.set_ylim(ymin - dy, ymax + dy)
    ax.set_prop_cycle('color', cm(np.linspace(0, 1, N)))

    return ax,

def update_frame(i):
    ax.plot(walk[i:i+2, 0], walk[i:i+2, 1])
    ax.set_title(f"Step {i+1} / {N}")
    return ax,


anim = animation.FuncAnimation(fig, update_frame, frames=N-1, init_func=init_frame, 
                               blit=False, repeat=True, interval=1)

plt.show()
