import numpy as np
import matplotlib.pyplot as plt

# Boundary conditions
Tnorth, Tsouth, Twest, Teast = 100, 20, 50, 50

# Set meshgrid
N, L = 64, 1.0
N2 = N*N
X, Y = np.meshgrid(np.linspace(0, L, N), np.linspace(0, L, N))

# RHS of the linear system
b = np.zeros((N, N))

# Add the boundary conditions in the RHS
b[N-1:, :] = Tnorth
b[:1, :]   = Tsouth
b[:, N-1:] = Teast
b[:, :1]   = Twest

# reshape the RHS
b = b.reshape(N2)

# Defines the matrix
A = np.zeros((N2, N2))
np.fill_diagonal(A, 4)
np.fill_diagonal(A[:-1, 1:], -1)
np.fill_diagonal(A[1:, :-1], -1)
np.fill_diagonal(A[:-N, N:], -1)
np.fill_diagonal(A[N:, :-N], -1)

# indices for north boundary
diag_indices = np.arange(N2-N, N2)
A[diag_indices, :] = 0     # 0 everywhere on each line
A[diag_indices, diag_indices] = 1   # 1 on the diagonal

# south
diag_indices = np.arange(0, N)
A[diag_indices, :] = 0     # 0 everywhere on each line
A[diag_indices, diag_indices] = 1   # 1 on the diagonal

# east
diag_indices = np.arange(N-1, N*N, N)
A[diag_indices, :] = 0     # 0 everywhere on each line
A[diag_indices, diag_indices] = 1   # 1 on the diagonal

# west
diag_indices = np.arange(N, N*N, N)
A[diag_indices, :] = 0     # 0 everywhere on each line
A[diag_indices, diag_indices] = 1   # 1 on the diagonal


# solve the linear system
T = np.linalg.solve(A, b).reshape((N, N))

plt.title("Temperature")
plt.contourf(X, Y, T)
plt.colorbar()
plt.show()
